/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002-2003 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef GTK_MTA_EDITOR_H
#define GTK_MTA_EDITOR_H

#include <gtk/gtkwidget.h>
#include "mta.h"


typedef struct XePGtkMTAEditor_s XePGtkMTAEditor;
typedef void (*XePGtkMTAEditorCb)(XePGtkMTAEditor *mtaeditor, gpointer data);

XePGtkMTAEditor *
a_GtkMTAEditor_new(GtkWidget *toplevel);

void
a_GtkMTAEditor_free(XePGtkMTAEditor *mtaeditor);

/*! Set the MTA to be edited or create a new one to edit
 *  \param mtaeditor the XePMTAEditor 
 *  \param mta the XePMTA to edit, or NULL to create a new one,
 *         in which case, the OK callback function should call
 *         a_GtkMTAEditor_get_mta() to get a pointer to the new
 *         XePMTA. If not, the newly created server will be 
 *         freeed.
 */
void
a_GtkMTAEditor_set_mta(XePGtkMTAEditor *mtaeditor, XePMTA *mta);

/*! Get the current mta
 *  \param mtaeditor the XePMTAEditor 
 *  This should be called from withing the OK callback
 *  to retrieve the eventually created MTA.
 *  This function return NULL if called twice in a row without
 *  a a_GtkMTAEditor_set_mta() between them.
 */
XePMTA *
a_GtkMTAEditor_get_mta(XePGtkMTAEditor *mtaeditor);

void
a_GtkMTAEditor_show(XePGtkMTAEditor *mtaeditor);

void
a_GtkMTAEditor_hide(XePGtkMTAEditor *mtaeditor);

/*! Set the function to be called whenever OK is clicked
 *  The callback function should call a_GtkMTAEditor_get_mta()
 *  to get the created MTA (when NULL is passed to a_GtkMTAEditor_set_mta())
 *  \param mtaeditor the XePMTAEditor 
 *  \param func the callback function to be called after user hits OK
 *  \param data the user data to pass to the function as second argument
 */
void
a_GtkMTAEditor_set_ok_callback(XePGtkMTAEditor *mtaeditor,
			       XePGtkMTAEditorCb func,
			       gpointer data);

void
a_GtkMTAEditor_set_cancel_callback(XePGtkMTAEditor *mtaeditor,
				   XePGtkMTAEditorCb func,
				   gpointer data);


#endif
