/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002-2003 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef MANAGER_H
#define MANAGER_H

#include <pthread.h>

#ifdef USE_PYTHON_WITH_THREAD_SUPPORT
#include <Python.h>
#endif

#include <libetpan/clist.h>

#include "xetpan-types.h"

#include "remote.h"

typedef struct XePManager_s XePManager;

struct XePManager_s
{
  XePRemote *remote;
  int fd_write;
  clist *pending_queue;
  int thread_count;
  int max_threads;
};

typedef enum
  {
    XEP_MANAGER_THREAD_STATE_NONE,
    XEP_MANAGER_THREAD_STATE_RUNNING,
    XEP_MANAGER_THREAD_STATE_IDLE
  } XePManagerThreadState;

typedef struct XePManagerThread_s XePManagerThread;

struct XePManagerThread_s
{
  pthread_mutex_t data_mutex;
  pthread_mutex_t queue_mutex;
  pthread_cond_t cond;
  pthread_t thread_id;
  XePManagerThreadState state;
  int cancel;
  clist *queue;
  int fd_write;

#ifdef USE_PYTHON_WITH_THREAD_SUPPORT
  PyThreadState *py_thread_state;
#endif
};

typedef enum
  {
    XEP_MANAGER_NONE,
    XEP_MANAGER_CAPABILITY,
    XEP_MANAGER_CREATE,
    XEP_MANAGER_DELETE,
    XEP_MANAGER_RENAME,
    XEP_MANAGER_SUBSCRIBE,
    XEP_MANAGER_UNSUBSCRIBE,
    XEP_MANAGER_LIST,
    XEP_MANAGER_LSUB,
    XEP_MANAGER_STATUS,
    XEP_MANAGER_APPEND,
    XEP_MANAGER_CLOSE,
    XEP_MANAGER_EXPUNGE,
    XEP_MANAGER_SEARCH,
    XEP_MANAGER_FETCH_FIELDS,
    XEP_MANAGER_GET_BODYSTRUCTURE,
    XEP_MANAGER_MIME_FIND_DEFAULT,
    XEP_MANAGER_FETCH_PART,
    XEP_MANAGER_FETCH,
    XEP_MANAGER_FETCH_BY_UID,
    XEP_MANAGER_FLAGS_CHANGE,
    XEP_MANAGER_LIST_FLAGS_CHANGE,
    XEP_MANAGER_STORE,
    XEP_MANAGER_COPY,
    XEP_MANAGER_PROCESSING,
    XEP_MANAGER_SEARCHING,
    XEP_MANAGER_UPDATE,
    XEP_MANAGER_CHECK,
    XEP_MANAGER_PROCESS,
    XEP_MANAGER_SMTP_SEND,
    XEP_MANAGER_DISCONNECT,
    XEP_MANAGER_MAILBOX_DISCONNECT,
    XEP_MANAGER_EXIT,
    XEP_MANAGER_DONE,
    XEP_MANAGER_EXIT_DONE
  } XePManagerMsgType;

typedef void (*XePManagerMsgCb)(void *data, void *arg_out);

typedef struct XePManagerMsg_s XePManagerMsg;

struct XePManagerMsg_s
{
  XePManagerMsgType type;
  void *source;
  XePManagerThread *target; /**< The target thread */
  void *arg_in;
  void *arg_out;
  clist *callbacks;
  XetPan *xetpan;
  XePManagerMsgType *state;
  int flush_old;
};

XePManagerMsg *
a_Manager_msg_new(void);

void
a_Manager_msg_free(XePManagerMsg *msg);

void
a_Manager_msg_set_type(XePManagerMsg *msg, XePManagerMsgType type);

/**! Cancel older messages by not executing them. Beware that their
 *   callbacks will nevertheless be called because they may be important
 *   for doing some cleanups
 */

void
a_Manager_msg_flush_old(XePManagerMsg *msg);

XePManagerMsgType
a_Manager_msg_get_type(XePManagerMsg *msg);

void
a_Manager_msg_set_source(XePManagerMsg *msg, void *source);

void *
a_Manager_msg_get_source(XePManagerMsg *msg);

void
a_Manager_msg_set_target(XePManagerMsg *msg, void *target);

void *
a_Manager_msg_get_target(XePManagerMsg *msg);

void
a_Manager_msg_set_source_target(XePManagerMsg *msg,
				void *source, 
				void *target);

void
a_Manager_msg_add_callback(XePManagerMsg *msg,
			   XePManagerMsgCb callback,
			   void *data);

void
a_Manager_msg_set_arg_in(XePManagerMsg *msg, void *arg_in);

void *
a_Manager_msg_get_arg_in(XePManagerMsg *msg);

void
a_Manager_msg_set_arg_out(XePManagerMsg *msg, void *arg_out);

void *
a_Manager_msg_get_arg_out(XePManagerMsg *msg);

void
a_Manager_msg_set_args(XePManagerMsg *msg, void *arg_in, void *arg_out);

XePManagerThreadState
a_Manager_thread_get_state_safe(XePManagerThread *thread);

int
a_Manager_thread_data_is_locked(XePManagerThread *thread);

void
a_Manager_msg_set_state_holder(XePManagerMsg *msg, 
			       XePManagerMsgType *state_holder);

void
a_Manager_msg_submit(XePManagerMsg *msg);

void 
a_Manager_init(XetPan *xetpan);

void
a_Manager_quit(XetPan *xetpan);

#endif
