/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PREFERENCES_H
#define PREFERENCES_H

#include "cntree.h"

typedef enum
  {
    XEP_PREFS_DEFAULT_CHARSET,

    XEP_PREFS_EMPTY_SUBJECT,
    XEP_PREFS_REPLY_REGEXP,
    XEP_PREFS_REPLY_PREFIX,
    XEP_PREFS_FORWARD_REGEXP,
    XEP_PREFS_FORWARD_PREFIX,
    XEP_PREFS_N
  } XePPreferencesToken;

typedef struct XePPreferences_s XePPreferences;
struct XePPreferences_s
{
  void *prefs[XEP_PREFS_N];
};

/*! Create user preferences from a parsed tree
 *  \param ntree an LXML tree containing user preferences
 *  \return a XePPreferences structure (global data). If ntree is NULL
 *          return default preferences.
 */
XePPreferences *
a_Preferences_create_from_tree(cntree *ntree);

void
a_Preferences_free(XePPreferences *preferences);

void *
a_Preferences_get(XePPreferencesToken token);

#define PREFS_DEFAULT_CHARSET() ((const char *) a_Preferences_get(XEP_PREFS_DEFAULT_CHARSET))

char *
a_Preferences_reply_subject_new(const char *subject);

char *
a_Preferences_forward_subject_new(const char *subject);

#endif
