/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002, 2003 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef VMESSAGE_H
#define VMESSAGE_H

#include <libetpan/libetpan.h>

#include "folder-types.h"
#include "message-types.h"

typedef struct XePVMessage_s XePVMessage;
struct XePVMessage_s
{
  XePFolder *folder;
  mailmessage *msg;
  clist *vfolders;
};

/*! Create a virtual message from a real message and
 *  register it in the virtual message hash tables 
 */

XePVMessage *
a_VMessage_new_from_message(XePMessage *message);

/*! Unregister the virtual message and release resources
 */
void
a_VMessage_free(XePVMessage *vmessage);

XePVMessage *
a_VMessage_get_from_uid(XePFolder *folder, const char *uid);

/*! Add the given vfolder to the list of vfolders where the message 
 *  is seen
 */
void
a_VMessage_add_to_folder(XePVMessage *vmessage, XePFolder *folder);

/*! Remove the given vfolder from the list of vfolders
 *  \return -1 if there is some vfolders that still reference it,
 *             or 0 if the vmessage is not contained in any vfolder
 *             anymore.
 */
int 
a_VMessage_remove_from_folder(XePVMessage *vmessage, XePFolder *folder);


/*! Add a reference to a VFolder in VMessage.
 *  This will add the vfolder to the list of vfolders containing
 *  the vmessage. 
 *  \param vmessage a non-NULL XePVMessage
 *  \param folder a non-NULL folder with a non-NULL id. The folder
 *         must be a virtual folder. No check is done.
 */
void
a_VMessage_ref_folder(XePVMessage *vmessage, XePFolder *folder);

/*! Remove a reference to a VFolder in a VMessage.
 *  This will remove the vfolder from the list of vfolders containing
 *  the vmessage
 *  If no more virtual folders are referencing the message, it is freeed.
 *  \param vmessage a non-NULL XePVMessage
 *  \param folder a non-NULL XePFolder with a non-NULL id. The
 *         folder must be a virtual folder. No check is done.
 *  \return 0 if folder removed or if the folder was not referenced,
 *          -1 if folder was removed and the vmessage was freeed because
 *          no more virtual folders are referencing it.
 */
int
a_VMessage_unref_folder(XePVMessage *vmessage, XePFolder *folder);

#endif
